package gvprojects.igel.model;

/**
 * An Igel Argern game board
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 1:38 PM
// (C) Zachary Kurmas 2011

class Board {

   private Cell[][] cells;
   private static final int[] DEFAULT_BLOCK_COLUMNS = {3, 6, 4, 5, 2, 7};

   /**
    * Constructor
    *
    * @param numRows    the number of rows on the board
    * @param numColumns the number of columns on the board
    * @param game       the game
    * @throws IllegalArgumentException if {@code numRows < 1}, {@code numColumns < 1}, or {@code game == null}
    */
   public Board(int numRows, int numColumns, IgelGameKurmas.ObstacleTypeKurmas obstacleType, IgelGameKurmas game) {
      if (numRows < 1 || numColumns < 1) {
         throw new IllegalArgumentException("Invalid number of rows or columns.");
      }
      if (game == null) {
         throw new IllegalArgumentException("Game may not be null");
      }
      cells = new Cell[numRows][numColumns];

      // If the bord has more than the standard number of rows, then randomly choose the locations of the obstacles
      // in the extra rows.
      int[] blockColumns = DEFAULT_BLOCK_COLUMNS;
      if (numRows >= blockColumns.length) {
         blockColumns = new int[numRows];
         System.arraycopy(DEFAULT_BLOCK_COLUMNS, 0, blockColumns, 0, DEFAULT_BLOCK_COLUMNS.length);
         for (int i = DEFAULT_BLOCK_COLUMNS.length; i < numRows; i++) {
            blockColumns[i] = ((int)(Math.random() * (numColumns() - 2)) + 1);
         }
      }

      // Create the cells on the board.
      for (int row = 0; row < numRows; row++) {
         for (int column = 0; column < numColumns - 1; column++) {

            // If this is the column for this row that should contain an obstacle,
            // then place an obstacle cell there.  Otherwise,
            // place a normal cell
            if (column == blockColumns[row]) {
               cells[row][column] = obstacleType.make(row, column, game);
            } else {
               cells[row][column] = new NormalCell(row, column, game);
            }
         }
         // The rightmost column (i.e., the finish line) contain special FinishCells
         // that don't allow hedgehogs to leave.
         cells[row][numColumns - 1] = new FinishCell(row, numColumns - 1, game);
      }
   }

   private void validateCellAccess(int row, int column) {
      if (row < 0 || row >= cells.length) {
         throw new IllegalArgumentException("Invalid row:  " + row);
      }
      if (column < 0 || column >= cells[0].length) {
         throw new IllegalArgumentException("Invalid column:  " + column);
      }
   }


   /**
    * Return the {@link Cell} at the specified location
    *
    * @param row    the row (indexed beginning at 0)
    * @param column the column (indexed beginning at 0)
    * @return the {@link Cell} at the specified location
    * @throws IllegalArgumentException if {@code row} or {@code column} are out of range.
    */
   public Cell getCell(int row, int column) {
      validateCellAccess(row, column);
      return cells[row][column];
   }

   public int numRows() {
      return cells.length;
   }

   public int numColumns() {
      return cells[0].length;
   }

   /**
    * Make an <em>initial</em> placement of one of {@code player}'s tokens in the specified row.
    *
    * @param row    the row
    * @param player the player
    * @throws IllegalArgumentException if {@code row} is not on the board.
    */
   public void placeToken(int row, int player) {
      validateCellAccess(row, 0);
      cells[row][0].push(player);
   }

   /**
    * Move the hedgehog at [{@code fromRow}, {@code fromColumn}] to [{@code fromRow}, {@code fromColumn}]
    *
    * @param fromRow    source row
    * @param fromColumn source column
    * @param toRow      target row
    * @param toColumn   target column
    * @throws java.util.NoSuchElementException
    *          if the source cell contains no hedgehog.
    */
   public void moveToken(int fromRow, int fromColumn, int toRow, int toColumn) {
      validateCellAccess(fromRow, fromColumn);
      validateCellAccess(toRow, toColumn);
      int player = cells[fromRow][fromColumn].pop();
      cells[toRow][toColumn].push(player);
   }
}
