package gvprojects.igel.model;


import java.util.Stack;

/**
 * A Cell on an Igel Argern board.
 *
 * @author Zachary Kurmas
 */
// (C) 2009 Zachary Kurmas
// Created Jul 20, 2009
abstract class Cell implements ICellState {

   // The stack of players currently in this cell
   protected Stack<Integer> players;


   private int row;
   private int col;
   private IgelGameKurmas game;

   /**
    * Constructor
    *
    * @param row    the row to which this Cell is assigned
    * @param column the column to which this Cell is assigned
    * @param game   the game
    * @throws IllegalArgumentException if {@code game == null)}
    */
   public Cell(int row, int column, IgelGameKurmas game) {
      if (game == null) {
         throw new IllegalArgumentException("game may not be null");
      }

      players = new Stack<Integer>();
      this.row = row;
      this.col = column;
      this.game = game;
   }

   public int getRow() {
      return row;
   }

   public int getColumn() {
      return col;
   }

   public int size() {
      return players.size();
   }

   /**
    * Returns player at the top of the stack.
    *
    * @return player at the top of the stack.
    * @throws java.util.NoSuchElementException
    *          if there are no players in the stack
    */
   public int peek() {
      return players.peek();
   }


   /**
    * Returns {@code true} if the given player may enter the cell.
    *
    * @param player the player
    * @return {@code true} if the given player may enter the cell.
    */
   public abstract boolean canEnter(int player);

   /**
    * Returns {@code true} if the given player may leave the cell.
    *
    * @return {@code true} if the given player may leave the cell.
    */
   public abstract boolean canLeave();


   /**
    * Returns true if any player is allowed to enter this cell
    *
    * @param numPlayers the number of players
    * @return {@code true} if any player may enter, {@code false} otherwise.
    */
   public boolean canAnybodyEnter(int numPlayers) {
      for (int i = 0; i < numPlayers; i++) {
         if (canEnter(i)) {
            return true;
         }
      }
      return false;
   }


   /**
    * Returns the player at the given depth (0 is the top of the stack).
    *
    * @param depth to be checked.
    * @return the player at the specified depth.
    */
   public int playerAtDepth(int depth) {
      return players.elementAt(players.size() - depth - 1);
   }

   /**
    * Returns a {@code String} uniquely identifying the type of this cell.
    *
    * @return a {@code String} uniquely identifying the type of this cell.
    */
   abstract public String type();

   /**
    * Returns an array listing all the players currently in the stack (0 is the
    * top of the stack).
    *
    * @return an array listing all the players currently in the stack (0 is the
    *         top of the stack).
    */
   protected int[] getAllPlayers() {
      int[] answer = new int[players.size()];
      for (int i = 0; i < players.size(); i++) {
         answer[i] = playerAtDepth(i);
      }
      return answer;
   }

   /**
    * Removes and returns the player at the top of the sack.
    *
    * @return the player at the top of the sack.
    * @throws IllegalMoveException if there are no players in the square or if player may not
    *                              leave square.
    */
   public int pop() throws IllegalMoveException {
      if (players.size() == 0) {
         throw new IllegalMoveException("No players in this Cell");
      }
      if (canLeave()) {
         return players.pop();
      } else {
         throw new IllegalMoveException(
               "Player at the top of the stack may not leave this square!");
      }
   }

   /**
    * Adds the player to the square
    *
    * @param p player to add.
    * @throws IllegalMoveException if the player may not enter the square.
    */
   public void push(int p) throws IllegalMoveException {
      if (canEnter(p)) {
         players.push(p);
      } else {
         throw new IllegalMoveException("Player " + p
               + " may not enter this square");
      }
   }

   /**
    * Return the game in which this cell is contained.
    *
    * @return the game in which this cell is contained.
    */
   protected IIgelGame getGame() {
      return game;
   }
} // end Cell
