package gvprojects.igel.model;

/**
 * A game of Igel Argern
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 10:48 AM
// (C) Zachary Kurmas 2011

public interface IIgelGame {

   /**
    * Listens for game events
    *
    * @author Zachary Kurmas
    */
   interface GameEventListener {

      /**
       * Called when message changes
       *
       * @param newMessage the new message
       */
      void messageChanged(String newMessage);

      /**
       * Reports a die roll
       *
       * @param newValue the new value on the die
       */
      public void roll(int newValue);

      /**
       * Reports a phase change
       *
       * @param newPhase the new {@link Phase}
       */
      public void phaseChanged(Phase newPhase);

      /**
       * Reports that the game has been won
       *
       * @param player the player that won
       */
      void announceWin(int player);


      /**
       * Report that there are no tokens to move in {@code row}
       *
       * @param row the row specified by the die.
       */
      void noForwardMove(int row);
   }


   /**
    * returns {@code true} if the current player can place a hedgehog in {@code row} (applies to initial hedgehog
    * placement only).
    *
    * @param row the desired row
    * @return {@code true} if the current player can place a hedgehog in {@code row}, {@code false} otherwise.
    * @throws IllegalArgumentException if {@code row} is not valid.
    * @throws IllegalMoveException     if the game isn't currently in the {@link Phase#SETUP} phase
    */
   boolean canPlaceHedgehog(int row);

   /**
    * Make an initial placement of one of the current player's hedgehogs in {@code row} and increment the current player.
    *
    * @param row the row
    * @throws IllegalArgumentException if {@code row} is not valid.
    * @throws IllegalMoveException     if the game isn't currently in the {@link Phase#SETUP} phase
    */
   void placeHedgehog(int row);


   /**
    * get a read-only view of the game's current state.
    *
    * @return a read-only view of the game's current state.
    */
   IGameState getState();

   /**
    * Add a {@link GameEventListener}
    *
    * @param listener the {@code MessageChangeListener} to add
    */
   void addGameEventListener(GameEventListener listener);


   /**
    * Return {@code true} if the current player can slide the hedgehog at [{@code fromRow}, {@code fromColumn}]
    *
    * @param fromRow    the current row of the player's hedgehog.
    * @param fromColumn the current column of the player's hedgehog.
    * @return {@code true} if the current player can slide the hedgehog at [{@code fromRow}, {@code fromColumn}].
    * @throws IllegalArgumentException if {@code row} is not valid.
    * @throws IllegalMoveException     if the game isn't currently in the {@link Phase#SIDEWAYS} phase
    */
   boolean canSlideHedgehog(int fromRow, int fromColumn);

   /**
    * Return {@code true} if the current player can slide the hedgehog at [{@code fromRow},
    * {@code fromColumn}] to   [{@code toRow}, {@code fromColumn}].
    *
    * @param fromRow    the current row of the player's hedgehog.
    * @param fromColumn the current column of the player's hedgehog.
    * @param toRow      the destination row
    * @return {@code true} if the current player can slide the hedgehog  at [{@code fromRow},
    *         {@code fromColumn}] to   [{@code toRow}, {@code fromColumn}]
    * @throws IllegalArgumentException if {@code row} is not valid.
    * @throws IllegalMoveException     if the game isn't currently in the {@link Phase#SIDEWAYS} phase
    */
   boolean canSlideHedgehog(int fromRow, int fromColumn, int toRow);


   /**
    * Move the current player's hedgehog from row {@code fromRow} to {@code toRow}
    *
    * @param fromRow    the current row of the player's hedgehog.
    * @param fromColumn the current column of the player's hedgehog.
    * @param toRow      the destination row.  @throws IllegalArgumentException if {@code row} is not valid.
    * @throws IllegalMoveException if the game isn't currently in the {@link Phase#SIDEWAYS} phase,
    *                              or of the requested move is not allowed.
    */
   void slideHedgehog(int fromRow, int fromColumn, int toRow);


   /**
    * Return whether the the top hedgehog at [{@code fromRow},
    * {@code fromColumn}] can be moved forward.   Note, the implementation in {@code IgelGameKurmas} checks only that
    * the hedgehog at [{@code fromRow},
    * {@code fromColumn}] may be moved.  It does not check whether the hedgehog may move into [{@code fromRow},
    * {@code fromColumn} + 1]
    *
    * @param fromRow    the row
    * @param fromColumn the column
    * @return {@code true} if the the top hedgehog at [{@code fromRow},
    *         {@code fromColumn}] can be moved forward.
    * @throws IllegalMoveException     if the game isn't currently in the {@link Phase#MOVE} phase.
    * @throws IllegalArgumentException if {@code fromRow} or {@code fromColumn} isn't valid.
    */
   boolean canMoveHedgehog(int fromRow, int fromColumn);


   /**
    * Return whether the the top hedgehog at [{@code fromRow},
    * {@code fromColumn}] can be moved to column {@code toColumn}.
    *
    * @param fromRow    the row
    * @param fromColumn the column
    * @param toColumn   the proposed new column
    * @return {@code true} if the the top hedgehog at [{@code fromRow},
    *         {@code fromColumn}] can be moved to column {@code toColumn}
    * @throws IllegalMoveException     if the game isn't currently in the {@link Phase#MOVE} phase.
    * @throws IllegalArgumentException if {@code fromRow} or {@code fromColumn} isn't valid.
    */
   boolean canMoveHedgehog(int fromRow, int fromColumn, int toColumn);

   /**
    * Move the top hedgehog at [{@code fromRow},
    * {@code fromColumn}]  to column {@code toColumn}.
    *
    * @param fromRow    the row
    * @param fromColumn the column
    * @param toColumn   the proposed new column
    * @throws IllegalMoveException if the game isn't currently in the {@link Phase#MOVE} phase,
    *                              or if the proposed move isn't legal.
    */
   void moveHedgehog(int fromRow, int fromColumn, int toColumn);

   /**
    * Pass the current user's sideways move.
    *
    * @throws IllegalMoveException if the game isn't currently in the {@link Phase#SIDEWAYS} phase.
    */
   void passSidewaysMove();

   /**
    * Set the value of the die rolls.  If {@code i} is {@code null}, roll a random value.  If {@code i} is not {@code
    * null}, then the die should return {@code i}
    *
    * @param i fix the die roll at {@code i}, or roll normally if {@code i} is {@code null}
    */
   void setNextRolls(Integer i);
}
