package gvprojects.igel.model;

import java.io.InputStream;
import java.util.*;

/**
 * Game engine for Igel Argern
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 10:48 AM
// (C) Zachary Kurmas 2011

public class IgelGameKurmas implements IIgelGame, IGameState {

   /**
    * Enum of obstacle types recognized by {@code IgelGameKurmas}
    */
    enum ObstacleTypeKurmas {
      CONCRETE_BLOCK, DEEP_PIT;

      /**
       * Make a cell of the type associated with this enum element
       *
       * @param row    the row at which this cell will be placed
       * @param column the column at which this cell will be placed
       * @param game   the game engine
       * @return a new cell of the specified type
       */
      Cell make(int row, int column, IgelGameKurmas game) {
         switch (this) {
            case CONCRETE_BLOCK:
               return new ConcreteBlock(row, column, game);
            case DEEP_PIT:
               return new DeepPitCell(row, column, game);
            default:
               throw new IllegalArgumentException("Unrecognized Cell type:  " + this);
         }
      }

      // Map strings to enum objects
      private static Map<String, ObstacleTypeKurmas> names = new HashMap<String, ObstacleTypeKurmas>();

      // initialize the map
      static {
         names.put("ConcreteBlock", CONCRETE_BLOCK);
         names.put("Concrete Block", CONCRETE_BLOCK);
         names.put("DeepPit", DEEP_PIT);
         names.put("Deep Pit", DEEP_PIT);
      }

      /**
       * Returns the {@link ObstacleTypeKurmas} that corresponds to the given string
       *
       * @param name the name of the desired obstacle type
       * @return the {@link ObstacleTypeKurmas} that corresponds to the given string
       */
      public static ObstacleTypeKurmas make(String name) {
         if (names.containsKey(name)) {
            return names.get(name);
         } else {
            throw new IllegalArgumentException("Unrecognized Obstacle Type:  " + name);
         }
      }
   } // end Obstacle Type

   private int numPlayers;
   private int hedgehogsPerPlayer;
   private Phase gamePhase;
   private Board gameBoard;
   private int currentPlayer;
   private int hedgehogsPlaced;

   private String currentMessage = "";

   // I included a list of changeListeners here for flexibility. I suspect that, as a practical matter,
   // this can really just be a single GameEventListener object.  I don't see students ever adding more than one listener.
   private Collection<GameEventListener> changeListeners = new HashSet<GameEventListener>();

   private Die die;
   private static final int NONE = -1;

   private Integer rollValue = null;

   /**
    * Constructor
    *
    * @param parameters an object containing all of the game parameters (num players, number of hedgehogs, etc.)
    * @throws IllegalArgumentException if {@code parameters} is null, or any parameter has an illegal value.
    */
   public IgelGameKurmas(IgelGameParameters parameters) {

      //
      // Validate / sanity check parameters
      //

      if (parameters == null) {
         throw new IllegalArgumentException("Parameters may not be null.");
      }

      if (parameters.numPlayers < 2) {
         throw new IllegalArgumentException("There must be at least two players");
      }

      if (parameters.numRows < 1) {
         throw new IllegalArgumentException("There must be at least one row.");
      }

      if (parameters.numColumns < 3) {
         throw new IllegalArgumentException("There must be at least three columns.");
      }

      if (parameters.hedgehogsPerPlayer < 1) {
         throw new IllegalArgumentException("Each player must have at least 1 hedgehog.");
      }

      if (parameters.hedgehogsToWin < 1) {
         throw new IllegalArgumentException("A player must have at least one hedgehog cross the finish line in order to win.");
      }

      if (parameters.hedgehogsToWin > parameters.hedgehogsPerPlayer) {
         throw new IllegalArgumentException("It doesn't make sense to require a player to " +
               "have more hedgehogs cross the finish line than he has hedgehogs!");
      }


      //
      // Initialize object.
      //

      this.numPlayers = parameters.numPlayers;
      this.hedgehogsPerPlayer = parameters.hedgehogsPerPlayer;
      this.gameBoard = new Board(parameters.numRows, parameters.numColumns, ObstacleTypeKurmas.make(parameters.obstacleType), this);

      changePhase(Phase.SETUP);
      this.currentPlayer = 0;
      this.hedgehogsPlaced = 0;

      die = new Die(parameters.numRows, new Random());
      rollDie();
      setPlaceHedgehogMessage();
   }

   /**
    * Constructor allowing configuration from a file (or other stream source).
    *
    * <p>
    * Note: Not all {@code parameters} may be used.  Typically, values specified in {@code input} override those
    * values in {@code parameters}
    * </p>
    *
    * @param parameters game parameters specified by the current user (e.g., from the configuration dialog).
    * @param input      game parameters contained in a file (or other data stream).
    * @throws IllegalArgumentException if any parameter is null or has an illegal value
    */
   public IgelGameKurmas(IgelGameParameters parameters, InputStream input) {
      throw new IllegalArgumentException("This method not implemented yet");
   }


   // From IGameState
   public int numRows() {
      return gameBoard.numRows();
   }

   // From IGameState
   public int numColumns() {
      return gameBoard.numColumns();
   }

   // From IGameState
   public int numPlayers() {
      return numPlayers;
   }

   // From IGameState
   public int hedgehogsPerPlayer() {
      return hedgehogsPerPlayer;
   }

   // From IGameState
   public Phase phase() {
      return gamePhase;
   }

   // From IGameState
   public int currentPlayer() {
      return currentPlayer;
   }

   /**
    * return the {@link ICellState} object at the given location
    *
    * @param row    the row (indexed beginning at 0)
    * @param column the column (indexed beginning at 0)
    * @return the {@link ICellState} object at the given location
    * @throws IllegalArgumentException if {@code row} or {@code column} is invalid
    */
   public ICellState getCell(int row, int column) {
      return gameBoard.getCell(row, column);
   }

   // From IIgelGame
   public boolean canPlaceHedgehog(int row) {
      if (phase() != Phase.SETUP) {
         throw new IllegalMoveException("May place hedgehogs only in the Setup phase.");
      }

      // How high will this stack be once the hedgehog is placed?
      int proposed_height = gameBoard.getCell(row, 0).size() + 1;


      // If any other stack has a height of proposed_height - 2 or less, then
      // that stack must be increased before this stack can be.
      for (int i = 0; i < gameBoard.numRows(); i++) {
         if (gameBoard.getCell(i, 0).size() < proposed_height - 1) {
            setMessage("You may not place a hedgehog in row " + (i + 1) +
                  " until all other rows have a height of " + (proposed_height - 1));
            return false;
         }
      }

      setPlaceHedgehogMessage();
      return true;
   }

   // From IIgelGame
   public IGameState getState() {
      // The IGameState interface has accessor methods only.  Thus, returning yourself as an IGameState object is one
      // way to assure that the calling method can view, but not modify, the game state.
      return this;
   }

   // From IIgelGame
   public void placeHedgehog(int row) {
      if (!canPlaceHedgehog(row)) {
         throw new IllegalMoveException("Can't place hedgehog here!");
      }

      gameBoard.placeToken(row, currentPlayer);

      // Increment the current player (i.e., make it the next player's turn)
      incrementPlayer();

      // If we have placed all the pieces, move on to the next phase.
      this.hedgehogsPlaced++;
      if (this.hedgehogsPlaced == this.hedgehogsPerPlayer * this.numPlayers) {
         beginSidewaysPhase();
      } else {
         setPlaceHedgehogMessage();
      }
   }

   private void beginSidewaysPhase() {
      changePhase(Phase.SIDEWAYS);
      setSidewaysMessage();
      rollDie();
   }

   // make it the next player's turn
   private void incrementPlayer() {
      currentPlayer++;
      currentPlayer %= numPlayers;
   }

   // From IIgelGame
   public void addGameEventListener(GameEventListener listener) {
      changeListeners.add(listener);
   }


   private void rollDie() {
      if (rollValue == null) {
         die.roll();
      } else {
         die.setValue(rollValue);
      }
      for (GameEventListener listener : changeListeners) {
         listener.roll(die.getValue());
      }
   }

   /**
    * return the current value of the die
    *
    * @return the current value of the die
    */
   public int currentDieValue() {
      return die.getValue();
   }

   /**
    * Get the current message.
    *
    * @return the current message
    */
   public String currentMessage() {
      return currentMessage;
   }

   private void changePhase(Phase newPhase) {
      gamePhase = newPhase;
      for (GameEventListener listener : changeListeners) {
         listener.phaseChanged(newPhase);
      }
   }

   // From IIgelGame
   public boolean canSlideHedgehog(int fromRow, int fromColumn) {
      if (phase() != Phase.SIDEWAYS) {
         throw new IllegalMoveException("May slide hedgehogs only in the Sideways phase.");
      }

      Cell cell = gameBoard.getCell(fromRow, fromColumn);

      // Can't slide a hedgehog out of an empty cell
      if (cell.size() == 0) {
         return false;
      }

      // Can only slide your own tokens
      if (cell.peek() != currentPlayer()) {
         setMessage("You may only slide your own tokens.");
         return false;
      }

      // Can't slide out of pits, cells at the finish line, etc.
      if (!cell.canLeave()) {
         setMessage("The hedgehog may not leave this cell.");
         return false;
      }

      return true;
   }

   // From IIgelGame
   public boolean canSlideHedgehog(int fromRow, int fromColumn, int toRow) {
      if (toRow >= gameBoard.numRows()) {
         throw new IllegalArgumentException("toRow value is too large");
      }

      if (!canSlideHedgehog(fromRow, fromColumn)) {
         return false;
      }

      if (fromRow == toRow) {
         setSidewaysMessage();
         return false;
      }

      if (fromRow != toRow + 1 && fromRow != toRow - 1) {
         setMessage("You can only slide tokens one column.");
         return false;
      }

      // Can't slide into concrete blocks
      Cell targetCell = gameBoard.getCell(toRow, fromColumn);
      if (!targetCell.canEnter(currentPlayer())) {
         setMessage("You may not enter this cell");
         return false;
      }

      setSidewaysMessage();
      return true;
   }

   /**
    * Slide the hedgehog at the top of the specified cell to the specified row.
    *
    * @param fromRow    the current row of the player's hedgehog.
    * @param fromColumn the current column of the player's hedgehog.
    * @param toRow      the destination row.
    * @throws IllegalArgumentException if any of the parameters are out of range.
    * @throws IllegalMoveException     if the move is not valid.
    */
   public void slideHedgehog(int fromRow, int fromColumn, int toRow) {
      if (!canSlideHedgehog(fromRow, fromColumn, toRow)) {
         throw new IllegalMoveException("Illegal Slide.");
      }

      gameBoard.moveToken(fromRow, fromColumn, toRow, fromColumn);
      changePhase(Phase.MOVE);
      setMoveMessage();
      checkForwardMovement();
   }

   // Sometimes a row contains no movable hedgehogs
   private void checkForwardMovement() {
       // The -1 accounts for the fact that the die is 1-based and the rows are 0-based
      if (!canMoveInRow(die.getValue() - 1)) {
         for (GameEventListener listener : changeListeners) {
            listener.noForwardMove(die.getValue());
         }
         endMovePhase();
      }
   }

   // From IIgelGame
   public boolean canMoveHedgehog(int fromRow, int fromColumn) {
      String message = canMoveHedgehogQuiet(fromRow, fromColumn);
      if (message == null) {
         return true;
      } else {
         setMessage(message);
         return false;
      }
   }

   private String canMoveHedgehogQuiet(int fromRow, int fromColumn) {
      // The -1 accounts for the fact that the die is 1-based and the rows are 0-based
      if (fromRow != die.getValue() - 1) {
         return ("You may only move a hedgehog in row " + die.getValue() + ".");
      }

      Cell cell = gameBoard.getCell(fromRow, fromColumn);
      if (cell.size() == 0) {
         return ("This cell is empty.");
      }

      if (!cell.canLeave()) {
         return ("The hedgehog is stuck in this cell.");
      }

      setMoveMessage();
      return null;
   }


   // From IIgelGame
   public boolean canMoveHedgehog(int fromRow, int fromColumn, int toColumn) {
      if (toColumn == fromColumn) {
         setMoveMessage();
         return false;
      }
      String message = canMoveHedgehogQuiet(fromRow, fromColumn, toColumn);
      if (message == null) {
         return true;
      } else {
         setMessage(message);
         return false;
      }
   }

   private String canMoveHedgehogQuiet(int fromRow, int fromColumn, int toColumn) {
      String message = canMoveHedgehogQuiet(fromRow, fromColumn);
      if (message != null) {
         return message;
      }

      if (toColumn != fromColumn + 1) {
         return ("You may only move one space forward");
      }

      Cell target = gameBoard.getCell(fromRow, toColumn);
      if (!target.canEnter(currentPlayer)) {
         return ("You can't enter that cell.");
      }
      return null;
   }

   // From IIgelGame
   public void moveHedgehog(int fromRow, int fromColumn, int toColumn) {
      if (!canMoveHedgehog(fromRow, fromColumn, toColumn)) {
         throw new IllegalMoveException("Can't move to this space!");
      }

      gameBoard.moveToken(fromRow, fromColumn, fromRow, toColumn);
      endMovePhase();
   }

   private void endMovePhase() {
      if (checkForWin() == NONE) {
         incrementPlayer();
         beginSidewaysPhase();
      }
   }


   // Determine if there is a move somewhere in the row.
   private boolean canMoveInRow(int row) {
      for (int i = 0; i < numColumns() - 1; i++) {
         if (canMoveHedgehogQuiet(row, i, i + 1) == null) {
            return true;
         }
      }
      return false;
   }


   private int checkForWin() {
      int[] finishCount = new int[numPlayers];
      for (int i = 0; i < numRows(); i++) {
         ICellState c = getCell(i, numColumns() - 1);
         for (int j = 0; j < c.size(); j++) {
            int player = c.playerAtDepth(j);
            if (++finishCount[player] == hedgehogsPerPlayer() - 1) {
               processWin(player);
               return player;
            }
         }
      }
      return NONE;
   }

   private void processWin(int player) {
      changePhase(Phase.DONE);
      for (GameEventListener listener : changeListeners) {
         listener.announceWin(player);
      }
   }


   // From IIgelGame
   public void passSidewaysMove() {
      if (gamePhase != Phase.SIDEWAYS) {
         throw new IllegalMoveException("Not currently in the Sideways phase.");
      }

      changePhase(Phase.MOVE);
      setMoveMessage();
      checkForwardMovement();
   }

   /**
    * Set the value of the die rolls.  If {@code i} is {@code null}, roll a random value.  If {@code i} is not {@code
    * null}, then the die should return {@code i}
    *
    * @param i fix the die roll at {@code i}, or roll normally if {@code i} is {@code null}
    */
   public void setNextRolls(Integer i) {
      rollValue = i;
   }

   private void setMoveMessage() {
      setMessage("Please move a hedgehog in row " + die.getValue() + " forward.");
   }

   private void setMessage(String message) {
      currentMessage =  "Player " + (currentPlayer + 1) + ": " +  message;
      for (GameEventListener listener : changeListeners) {
         listener.messageChanged(currentMessage);
      }
   }

   private void setPlaceHedgehogMessage() {
      setMessage("Please place a hedgehog.");
   }

   private void setSidewaysMessage() {
      setMessage("Please make a sideways move.");
   }

   /**
    * Make an initial placement of all hedgehogs in round-robin order.
    * This method is used during development only as a shortcut for testing.
    */
   public void quickLayout() {
      for (int i = 0; i < numPlayers() * hedgehogsPerPlayer(); i++) {
         placeHedgehog((i + numPlayers() / 2) % numRows());
      }
   }

}
