package gvprojects.igel.presenter;

import edu.gvsu.kurmasz.warszawa.util.BuildDate;
import gvprojects.igel.IgelArgern;
import gvprojects.igel.model.IGameState;
import gvprojects.igel.model.*;
import gvprojects.igel.view.DefaultIgelMenu;
import gvprojects.igel.view.IIgelView;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Date;

/**
 * Example presenter that (1) interacts with the view using clicks only, and (2) interacts with the model without
 * using "polling" callbacks.
 *
 * @author Zachary Kurmas
 */
// Created  1/16/12 at 8:55 AM
// (C) Zachary Kurmas 2012

public class IgelArgernClickOnly {

   /**
    * Callback announcing the end of a game.
    */
   public interface GameEndListener {
      void finished();
   }

   private IIgelView view;
   private IIgelGame model;
   private GameEndListener endListener;

   /**
    * Constructor
    *
    * @param view            the view
    * @param model           the model
    * @param gameEndListener a listener called when the game ends
    */
   public IgelArgernClickOnly(final IIgelView view, final IIgelGame model, final GameEndListener gameEndListener) {

      this.view = view;
      this.model = model;
      this.endListener = gameEndListener;


      // Add a MouseListener to every cell on the board.
      // Adding a separate listener to every cell on the board allows
      // us to figure out which cell was clicked without having to poll all the cells.
      for (int r = 0; r < model.getState().numRows(); r++) {
         for (int c = 0; c < model.getState().numColumns(); c++) {
            final int row = r;
            final int column = c;
            view.addMouseListener(r, c, new MouseAdapter() {
               public void mouseClicked(MouseEvent mouseEvent) {
                  cellClicked(row, column, mouseEvent);
               }
            });
         }
      }

      view.addPassSidewaysMovePressedListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            if (model.getState().phase() == Phase.SIDEWAYS) {
               model.passSidewaysMove();
               view.updateMessage(model.getState().currentMessage());
               checkForChangeToSidewaysPhase();
            }
         }
      });

      //
      // Set up the menu and its listeners.  The menu is in a separate class because I didn't want to put too many
      // menu specifics in the view interface.
      //
      DefaultIgelMenu menu = new DefaultIgelMenu();
      view.addMenu(menu.getMenuBar());

      menu.addExitListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            // exit the program if (1) The game is done, or (2) the game is not done, but the user responds affirmatively
            // to a confirmation dialog
            if (model.getState().phase() == Phase.DONE ||
                  DefaultIgelMenu.getPermission("Are you sure you want to abandon the game in progress and exit?")) {
               System.exit(0);
            }
         }
      });

      menu.addRestartListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            // restart the game if (1) The game is done, or (2) the game is not done, but the user responds affirmatively
            // to a confirmation dialog
            if (model.getState().phase() == Phase.DONE ||
                  DefaultIgelMenu.getPermission("Are you sure you want to abandon the game in progress and exit?")) {
               endListener.finished();
            }//To change body of created methods use File | Settings | File Templates.
         }
      });

      menu.addAboutListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            Date buildDate = BuildDate.getBuildDate(IgelArgern.class);
            String dateString = "";
            if (buildDate != null) {
               dateString = "\nSupport code built: " + buildDate;
            }

            String message = "Igel Argern\n" +
                  "View: " + view.getClass().getName() + "\n" +
                  "Model: " + model.getClass().getName() + dateString;
            view.displayDialogMessage(message, "About");
         }
      });
   }

   // Called when a cell is clicked.
   private void cellClicked(int row, int column, MouseEvent mouseEvent) {
      IGameState state = model.getState();
      switch (state.phase()) {
         case SETUP:
            placeInitialToken(row, column);
            break;
         case SIDEWAYS:
            moveSideways(row, column, mouseEvent);
            break;
         case MOVE:
            moveForward(row, column);
            break;
      }
   }

   // The die is rolled when the phase changes to SIDEWAYS
   private void checkForChangeToSidewaysPhase() {
      if (model.getState().phase() == Phase.SIDEWAYS) {
         view.updateDie(model.getState().currentDieValue());
      }
   }

   private void placeInitialToken(int row, int column) {
      if (model.canPlaceHedgehog(row)) {
         model.placeHedgehog(row);
         view.update(model.getState());
         checkForChangeToSidewaysPhase();
      } else {
         youCantDoThat();
      }
      view.updateMessage(model.getState().currentMessage());
   }

   private void moveSideways(final int row, final int column, MouseEvent event) {

      // If we can't slide this hedgehog, then don't even show the menu.
      if (!model.canSlideHedgehog(row, column)) {
         view.updateMessage(model.getState().currentMessage());
         return;
      }

      // Build the pop-up menu that allows the user to choose whether to slide the token up or down.
      // (I don't like having this GUI code in the presenter; but, its the best idea I have right now).
      class MyMenu extends JPopupMenu {
         JMenuItem up;
         JMenuItem down;

         public MyMenu() {
            if (row > 0) {
               up = new JMenuItem("Slide Up");
               up.addActionListener(new ActionListener() {
                  public void actionPerformed(ActionEvent actionEvent) {
                     moveSideways(row, column, row - 1);
                  }
               });
               add(up);
            }

            if (row < model.getState().numRows() - 1) {
               down = new JMenuItem("Slide Down");
               down.addActionListener(new ActionListener() {
                  public void actionPerformed(ActionEvent actionEvent) {
                     moveSideways(row, column, row + 1);
                  }
               });
               add(down);
            }
         }
      }

      MyMenu menu = new MyMenu();
      Component component = event.getComponent();

      // Display the menu below the cell
      menu.show(component, 0, component.getHeight());

      view.updateMessage(model.getState().currentMessage());
   }

   private void moveSideways(final int row, final int column, int toRow) {
      if (model.canSlideHedgehog(row, column, toRow)) {
         model.slideHedgehog(row, column, toRow);
         view.update(model.getState());
         checkForChangeToSidewaysPhase();
      } else {
         youCantDoThat();
      }
   }

   private void moveForward(int row, int column) {
      if (model.canMoveHedgehog(row, column, column + 1)) {
         model.moveHedgehog(row, column, column + 1);
         view.updateDie(model.getState().currentDieValue());
         view.update(model.getState());
         checkForWin();
      } else {
         youCantDoThat();
         view.clearProposedMove();
      }
      view.updateMessage(model.getState().currentMessage());
   }

   private void youCantDoThat() {
      view.displayDialogMessage(model.getState().currentMessage(), "You can't do that!");
   }

   private void checkForWin() {
      if (model.getState().phase() == Phase.DONE) {
         view.displayDialogMessage("Congratulations player " + model.getState().currentPlayer() + ". You won.",
               "We have a winner");
         endListener.finished();
      }
   }
}
