package gvprojects.igel.presenter;

import edu.gvsu.kurmasz.warszawa.util.BuildDate;
import gvprojects.igel.IgelArgern;
import gvprojects.igel.model.IGameState;
import gvprojects.igel.model.IIgelGame;
import gvprojects.igel.model.Phase;
import gvprojects.igel.view.CellActionListener;
import gvprojects.igel.view.DefaultIgelMenu;
import gvprojects.igel.view.IIgelView;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;

/**
 * Full/Fancy presenter for Igel Argern game.  This presenter supports drag-and-drop and does not show illegal moves.
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 10:51 AM
// (C) Zachary Kurmas 2011

public class IgelArgernFullPresenter {

   /**
    * Allows the presenter to announce that a game has ended.
    */
   public interface GameEndListener {
      void finished();
   }

   private IIgelView view;
   private IIgelGame model;

   /**
    * Constructor
    *
    * @param view        the view
    * @param model       the model
    * @param endListener listener to which presenter announces the end of the game
    */
   public IgelArgernFullPresenter(final IIgelView view, final IIgelGame model, final GameEndListener endListener) {

      this.view = view;
      this.model = model;


      // Add listeners for cell-based events in the view
      view.addCellActionListener(new CellActionListener() {
         public void cellEntered(int row, int column) {
            userEntersCell(row, column);
         }

         public void cellDragEntered(int row, int column, int sourceRow, int sourceColumn) {
            userDragsIntoCell(row, column, sourceRow, sourceColumn);
         }

         public void cellExited(int row, int column) {
            return;
         }

         public void cellClicked(int row, int column) {
            userClicksInCell(row, column);
         }

         public void cellPressed(int row, int column) {
            userPressesInCell(row, column);
         }

         public void cellReleased(int releasedRow, int releasedColumn, int pressedRow, int pressedColumn) {
            userReleasesInCell(releasedRow, releasedColumn, pressedRow, pressedColumn);
         }
      });


      view.addPassSidewaysMovePressedListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            passSidewaysMovePressed();
         }
      });
      view.enableSidewaysViewButton(false);

      model.addGameEventListener(new IIgelGame.GameEventListener() {
         public void messageChanged(String newMessage) {
            view.updateMessage(newMessage);
         }

         public void roll(int newValue) {
            view.updateDie(newValue);
         }

         public void phaseChanged(Phase newPhase) {
            view.enableSidewaysViewButton(newPhase == Phase.SIDEWAYS);
         }

         public void announceWin(int player) {
            view.displayDialogMessage("Congratulations, player " + (player + 1) + "!", "We have a winner!");
            endListener.finished();
         }

         public void noForwardMove(int row) {
            view.displayDialogMessage("None of the hedgehogs in row " + row + " can be moved.", "No moves!");
         }
      });


      //
      // Set up the menu and its listeners.  The menu is in a separate class because I didn't want to put too many
      // menu specifics in the view interface.
      //
      DefaultIgelMenu menu = new DefaultIgelMenu();
      view.addMenu(menu.getMenuBar());

      menu.addExitListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            // exit the program if (1) The game is done, or (2) the game is not done, but the user responds affirmatively
            // to a confirmation dialog
            if (model.getState().phase() == Phase.DONE ||
                  DefaultIgelMenu.getPermission("Are you sure you want to abandon the game in progress and exit?")) {
               System.exit(0);
            }
         }
      });

      menu.addRestartListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            // restart the game if (1) The game is done, or (2) the game is not done, but the user responds affirmatively
            // to a confirmation dialog
            if (model.getState().phase() == Phase.DONE ||
                  DefaultIgelMenu.getPermission("Are you sure you want to abandon the game in progress and exit?")) {
               endListener.finished();
            }//To change body of created methods use File | Settings | File Templates.
         }
      });

      menu.addAboutListener(new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            Date buildDate = BuildDate.getBuildDate(IgelArgern.class);
            String dateString = "";
            if (buildDate != null) {
               dateString = "\nSupport code built: " + buildDate;
            }

            String message = "Igel Argern\n" +
                  "View: " + view.getClass().getName() + "\n" +
                  "Model: " + model.getClass().getName() + dateString;
            view.displayDialogMessage(message, "About");
         }
      });


      // When writing and debugging the game, it gets annoying to have to lay out all 16 hedgehogs every time you
      // re-start the program.  So, I added a shortcut:
      //((IgelGameKurmas) model).quickLayout();
   }

   private static int topPlayerAt(IGameState state, int row, int column) {
      return state.getCell(row, column).playerAtDepth(0);
   }

   private void passSidewaysMovePressed() {
      model.passSidewaysMove();
   }

   private void userReleasesInCell(int row, int column, int fromRow, int fromColumn) {
      IGameState state = model.getState();

      // If the move is legal, make it.  Otherwise, just clear the proposed move.
      switch (state.phase()) {
         case SIDEWAYS:
            view.clearProposedMove();
            if (fromRow != row && model.canSlideHedgehog(fromRow, fromColumn, row)) {
               model.slideHedgehog(fromRow, fromColumn, row);
               view.update(model.getState());
            }
            break;
         case MOVE:
            view.clearProposedMove();
            if (fromColumn != column && model.canMoveHedgehog(fromRow, fromColumn, column)) {
               model.moveHedgehog(fromRow, fromColumn, column);
               view.update(model.getState());
            }

            break;
      }
   }

   private void userPressesInCell(int row, int column) {
      IGameState state = model.getState();
      switch (state.phase()) {
         case SIDEWAYS:
            if (model.canSlideHedgehog(row, column)) {
               view.setProposedMoveSource(row, column);
               view.setProposedMoveTarget(row, column, state.currentPlayer());
            }
            break;
         case MOVE:
            if (model.canMoveHedgehog(row, column)) {
               view.clearProposedMove();
               view.setProposedMoveSource(row, column);
               view.setProposedMoveTarget(row, column, topPlayerAt(state, row, column));
            }
            break;
      }
   }

   private void userDragsIntoCell(int row, int column, int sourceRow, int sourceColumn) {
      IGameState state = model.getState();
      switch (state.phase()) {
         case SIDEWAYS:
            if (!model.canSlideHedgehog(sourceRow, sourceColumn)) {
               break;
            }

            // If we moved back onto the original location or we are allowed to move to this location,
            // then show the proposed move.  Releasing on the same cell that was pressed effectively cancels the move.
            if (row == sourceRow ||
                  model.canSlideHedgehog(sourceRow, sourceColumn, row)) {
               view.setProposedMoveTarget(row, sourceColumn, state.currentPlayer());
            } else {
               view.setProposedMoveTarget(sourceRow, sourceColumn,
                     state.currentPlayer());
            }
            break;

         case MOVE:
            if (!model.canMoveHedgehog(sourceRow, sourceColumn)) {
               break;
            }
            int topPlayer = topPlayerAt(state, sourceRow, sourceColumn);
            if ((column == sourceColumn) ||
                  model.canMoveHedgehog(sourceRow, sourceColumn, column)) {
               view.setProposedMoveTarget(sourceRow, column, topPlayer);
            } else {
               view.setProposedMoveTarget(sourceRow, sourceColumn,
                     topPlayer);
            }
            break;
      } // end switch
   }


   private void userEntersCell(int row, int column) {
      IGameState state = model.getState();
      switch (state.phase()) {
         case SETUP:
            if (model.canPlaceHedgehog(row)) {
               view.setProposedMoveTarget(row, 0, state.currentPlayer());
            } else {
               view.clearProposedMove();
            }
            break;
      } // end switch
   }

   private void userClicksInCell(int row, int column) {
      IGameState state = model.getState();
      switch (state.phase()) {
         case SETUP:
            view.clearProposedMove();
            if (model.canPlaceHedgehog(row)) {
               model.placeHedgehog(row);
               view.update(model.getState());
            }
            break;
      }
   }
}
