package gvprojects.igel.view;


import gvprojects.igel.model.ICellState;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;


/**
 * A Swing {@code JPanel} representing a single square on the game board.  This class also keeps track of the last
 * cell entered and clicked.  Tracking this data allows the {@link CellActionListener} interface to provide data that
 * simplifies drag-and-drop.
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 11:46 AM
// (C) Zachary Kurmas 2011

public class CellPanel extends JPanel {

   public static final int NONE = -1;

   private ICellState state;
   private Artist artist;

   private List<CellActionListener> listeners = new ArrayList<CellActionListener>();

   // number of player proposing to move into this square
   private int proposedEntry = NONE;

   // do not show the top element in the cell.  (Used to illustrate proposed moves.)
   private boolean hideTop = false;


   // The Java Swing mouseReleased event occurs on the component over which the mouse was *pressed*,
   // not over the component over which the mouse was released.
   // This can make implementing drag-and-drop confusing.  To help alleviate this confusion, these variables
   // keep track of the last CellPanel entered and clicked in so that we can report that to the listener (which means
   // that the programmer doesn't have to keep track of it herself.)

   private static int lastPressedRow = NONE;
   private static int lastPressedColumn = NONE;
   private static int lastEnteredRow = NONE;
   private static int lastEnteredColumn = NONE;


   /**
    * Should the top hedgehog be hidden?  (Used to illustrate proposed moves.)
    *
    * @return {@code true} if the hedgehog at the top of the stack should be hidden.
    */
   public boolean hideTop() {
      return hideTop;
   }

   /**
    * Specify whether the hedgehog at the top of the stack should be hidden.
    *
    * @param hideTop whether to hide the top hedgehog.
    */
   public void setHideTop(boolean hideTop) {
      this.hideTop = hideTop;
      repaint();
   }


   /**
    * Constructor
    *
    * @param row    the row in which this {@code CellPanel} has been placed.
    * @param column the column in which this {@code CellPanel} has been placed.
    * @param state  a description of the {@code Cell} to be drawn.
    * @param artist the {@link Artist} responsible for drawing this cell.
    */
   public CellPanel(final int row, final int column, ICellState state, Artist artist) {
      super();

      if (artist == null) {
         throw new IllegalArgumentException("Artist may not be null.");
      }

      this.state = state;
      this.artist = artist;
      setBorder(new SoftBevelBorder(BevelBorder.RAISED));

      // Intercept mouse actions and translate them into CellActionListener actions.
      // The CellActionListener methods provide parameters that simplify drag-and-drop movements.
      // Presenters that use a CellActionListener need only set up one listener (as opposed to explicitly setting one
      // listener per cell).
      addMouseListener(new MouseListener() {
         public void mouseClicked(MouseEvent mouseEvent) {
            for (CellActionListener l : listeners) {
               l.cellClicked(row, column);
            }
         }

         public void mousePressed(MouseEvent mouseEvent) {
            lastPressedColumn = column;
            lastPressedRow = row;
            for (CellActionListener l : listeners) {
               l.cellPressed(row, column);
            }
         }

         public void mouseReleased(MouseEvent mouseEvent) {
            lastPressedRow = NONE;
            lastPressedColumn = NONE;

            for (CellActionListener l : listeners) {
               l.cellReleased(lastEnteredRow, lastEnteredColumn, row, column);
            }
         }

         public void mouseEntered(MouseEvent mouseEvent) {
            lastEnteredRow = row;
            lastEnteredColumn = column;
            for (CellActionListener l : listeners) {
               if (lastPressedRow == NONE) {
                  l.cellEntered(row, column);
               } else {
                  l.cellDragEntered(row, column, lastPressedRow, lastPressedColumn);
               }
            }
         }

         public void mouseExited(MouseEvent mouseEvent) {
            for (CellActionListener l : listeners) {
               l.cellExited(row, column);
            }
         }
      });
   }

   /**
    * Add a {@link CellActionListener}, which reports mouse events in the cell
    *
    * @param cellActionListener the listener.
    */
   public void addCellActionListener(CellActionListener cellActionListener) {
      listeners.add(cellActionListener);
   }

   @Override
   public void paintComponent(Graphics g) {
      super.paintComponent(g);
      getArtist().draw(this, g, state);
   }

   /**
    * Specify that {@code player} is proposing to enter this cell
    *
    * @param player the player
    */
   public void setProposedEntry(int player) {
      proposedEntry = player;
      repaint();
   }

   /**
    * Specify that no player is proposing to enter this cell
    */
   public void clearProposedEntry() {
      proposedEntry = NONE;
      repaint();
   }

   /**
    * returns the number of the player who is proposing to enter this square, or {@link CellPanel#NONE} if there are
    * no proposed entries.
    *
    * @return the number of the player who is proposing to enter this square, or {@link CellPanel#NONE}
    */
   public int getProposedEntry() {
      return proposedEntry;
   }

   /**
    * Return the {@code Artist} responsible for drawing this cell.
    *
    * @return the {@code Artist} responsible for drawing this cell.
    */
   protected Artist getArtist() {
      return artist;
   }
}
