package gvprojects.igel.view;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * A graphical representation of a six-sided die.
 *
 * This class is a modification of Scott Grissom's {@code GVDice} class.
 */
public class GVDiePanel extends JPanel {
   /**
    * current value of the die
    */
   private int myValue, displayValue;


   /**
    * current size in pixels
    */
   private int mySize;

   /**
    * dot size in pixels defined by overall die size
    */
   private int dotSize;

   /**
    * offset in pixels for the left row of dots
    */
   private int left;

   /**
    * offset in pixels for the right row of dots
    */
   private int right;

   /**
    * offset in pixels for the middle dot
    */
   private int middle;

   /**
    * delay for animation
    */
   private int DELAY;

   /**
    * default color of dice
    */
   private Color BACKGROUND = Color.white;

   /**
    * repeats for animation
    */
   private int NUM_ROLLS;

   /**
    * Timer for animation
    */
   private javax.swing.Timer myTimer;

   private int maxValue = 6;

   /**
    * constructor creates a die of specified size X size pixels
    *
    * @param size the length of each side in pixels
    */
   public GVDiePanel(int size, int mv) {
      // initialize the die and determine display characteristics
      maxValue = mv;
      mySize = size;
      dotSize = mySize / 5;
      int spacer = (mySize - (3 * dotSize)) / 4;
      left = spacer;
      right = mySize - spacer - dotSize;
      middle = (mySize - dotSize) / 2;
      setBackground(BACKGROUND);
      setForeground(Color.black);
      setSize(size, size);
      setPreferredSize(new Dimension(size, size));
      setMinimumSize(new Dimension(size, size));
      setMaximumSize(new Dimension(size, size));

      // create the fancy border
      Border raised = BorderFactory.createRaisedBevelBorder();
      Border lowered = BorderFactory.createLoweredBevelBorder();
      Border compound = BorderFactory.createCompoundBorder(raised, lowered);
      setBorder(compound);

      // set default values
      displayValue = myValue = 1;
      setDelay(250);
      setNumRolls(6);
      myTimer = new javax.swing.Timer(DELAY, new Animator());
   }

   /**
    * default constructor creates a die of size 100 X 100 pixels
    */
   public GVDiePanel() {
      this(100, 6);
   }

   /**
    * Set the die face to blank
    */
   public void setBlank() {
      displayValue = 0;
      repaint();
   }

   /**
    * Sets the color of the dots
    *
    * @param c a Java Color object such as Color.red
    */
   public void setForeground(Color c) {
      super.setForeground(c);
   }

   /**
    * Updates the image after obtaining a random value in the range 1 - 6.
    *
    * @param newValue the new value for the die
    */
   public void rollTo(int newValue) {
      myValue = newValue;

      // start the animated roll
      myTimer.restart();

   }

   /**
    * Set the delay in milliseconds between frames of the animation. Default
    * value is 250.
    *
    * @param msec milliseconds to delay
    */
   public void setDelay(int msec) {
      DELAY = 0;
      if (msec > 0)
         DELAY = msec;
   }

   /**
    * Set the number of rolls before stopping the animation. Default value is
    * 6.
    *
    * @param num number of rolls before stopping
    */
   public void setNumRolls(int num) {
      NUM_ROLLS = 0;
      if (num > 0)
         NUM_ROLLS = num;
   }

   /**
    * gets the current value of the die (1 - 6)
    *
    * @return the current value of the die
    */
   public int getValue() {
      return myValue;
   }

   /**
    * Display the current value of the die. Called automatically after rolling.
    * There is no need to call this method directly.
    *
    * @param g the graphics context for the panel
    */
   @SuppressWarnings("fallthrough")
   public void paintComponent(Graphics g) {
      super.paintComponent(g);

      // paint dots
      switch (displayValue) {
         case 1:
            g.fillOval(middle, middle, dotSize, dotSize);
            break;
         case 2:
            g.fillOval(left, left, dotSize, dotSize);
            g.fillOval(right, right, dotSize, dotSize);
            break;
         case 3:
            g.fillOval(middle, left, dotSize, dotSize);
            g.fillOval(middle, middle, dotSize, dotSize);
            g.fillOval(middle, right, dotSize, dotSize);
            break;
         case 5:
            g.fillOval(middle, middle, dotSize, dotSize);
            // fall through and paint four more dots
         case 4:
            g.fillOval(left, left, dotSize, dotSize);
            g.fillOval(left, right, dotSize, dotSize);
            g.fillOval(right, left, dotSize, dotSize);
            g.fillOval(right, right, dotSize, dotSize);
            break;
         case 6:
            g.fillOval(left, left, dotSize, dotSize);
            g.fillOval(left, middle, dotSize, dotSize);
            g.fillOval(left, right, dotSize, dotSize);
            g.fillOval(right, left, dotSize, dotSize);
            g.fillOval(right, middle, dotSize, dotSize);
            g.fillOval(right, right, dotSize, dotSize);
            break;
      }

   }


   /**
    * INNER class to roll the dice as an animation
    */
   private class Animator implements ActionListener {
      int count = 0;

      public void actionPerformed(ActionEvent e) {
         displayValue = (int) (Math.random() * 6) + 1;
         repaint();
         count++;
         // Should we stop rolling?
         if (count == NUM_ROLLS) {
            count = 0;
            myTimer.stop();
            displayValue = myValue;
            repaint();
         }
      }
   }
}
