package gvprojects.igel.view;

import gvprojects.igel.model.ICellState;
import gvprojects.igel.model.IGameState;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;
import java.awt.*;
import java.awt.event.MouseListener;
import java.util.Map;

/**
 * Contains the individual cells of an Igel Argern game.
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 11:21 AM
// (C) Zachary Kurmas 2011

class GameBoard extends JPanel {

   private CellPanel[][] cells;

   /**
    * Constructor
    *
    * @param gameState object describing the state of the game.
    * @param artistMap {@code Map} of cell types to the {@link Artist} that draws them.
    */
   public GameBoard(IGameState gameState, Map<String, Artist> artistMap) {
      int numRows = gameState.numRows();
      int numColumns = gameState.numColumns();
      setLayout(new GridLayout(numRows, numColumns + 1));
      cells = new CellPanel[numRows][numColumns];
      for (int row = 0; row < numRows; row++) {
         NumberPanel np = new NumberPanel(row + 1 + "");
         np.setBorder(new SoftBevelBorder(BevelBorder.RAISED));
         add(np);
         for (int col = 0; col < numColumns; col++) {
            ICellState state = gameState.getCell(row, col);
            Artist artist = artistMap.get(state.type());
            cells[row][col] = new CellPanel(row, col, state, artist);
            add(cells[row][col]);
         }
      }
   }

   /**
    * Add the given listener to each cell in the board.
    *
    * @param cellActionListener the listener
    */
   public void addCellActionListener(CellActionListener cellActionListener) {
      for (CellPanel[] row : cells) {
         for (CellPanel c : row) {
            c.addCellActionListener(cellActionListener);
         }
      }
   }

   private void validateCellAccess(int row, int column) {
      if (row < 0 || row > cells.length) {
         throw new IllegalArgumentException("Invalid row:  " + row);
      }
      if (column < 0 || column > cells[0].length) {
         throw new IllegalArgumentException("Invalid column:  " + column);
      }
   }

   /**
    * Show that {@code player} is proposing to enter cell [{@code row}, {@code cloumn}]
    *
    * @param row    the row
    * @param column the column
    * @param player the player number
    * @throws IllegalArgumentException if either {@code row} or {@code column} are out of range.
    */
   public void setProposedMoveTarget(int row, int column, int player) {
      validateCellAccess(row, column);
      for (CellPanel[] r : cells) {
         for (CellPanel cell : r) {
            cell.clearProposedEntry();
         }
      }
      cells[row][column].setProposedEntry(player);
   }

   /**
    * Cancels any proposed move.
    */
   public void clearProposedMove() {
      for (CellPanel[] r : cells) {
         for (CellPanel cell : r) {
            cell.setHideTop(false);
            cell.clearProposedEntry();
         }
      }
   }

   /**
    * Show that {@code player} is proposing to move from cell [{@code row}, {@code cloumn}]
    *
    * @param row    the row
    * @param column the column
    * @throws IllegalArgumentException if either {@code row} or {@code column} are out of range.
    */
   public void setProposedMoveSource(int row, int column) {
      validateCellAccess(row, column);
      clearProposedMove();
      cells[row][column].setHideTop(true);
      repaint();
   }

   /**
    * Add the specified listener to cell {@code [row, col]}.
    *
    * @param row      the row
    * @param col      the column
    * @param listener the listener
    * @throws IllegalArgumentException if either {@code row} or {@code column} are out of range.
    */
   public void addMouseListener(int row, int col, MouseListener listener) {
      validateCellAccess(row, col);
      cells[row][col].addMouseListener(listener);
   }
}
