package gvprojects.igel.view;

import gvprojects.igel.model.IGameState;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.util.HashMap;
import java.util.Map;

/**
 * Example view for IgelArgern
 *
 * @author Zachary Kurmas
 */
// Created  11/22/11 at 10:53 AM
// (C) Zachary Kurmas 2011

public class IgelViewKurmas implements IIgelView {

   private JFrame frame;
   private JLabel instructionLabel;
   private GameBoard gameBoard;

   private GVDiePanel die;

   private JButton passSidewaysMoveButton = new JButton("Pass Sideways Move");

   /**
    * Constructor
    *
    * @param gameState object describing the state of the game (The {@code IGameState} object describes the game
    *                  configuration:  board size, location of obstacles, obstacle type, etc.)
    * @param map       a map of {@code String}s to the {@link Artist}s that handle Cells of the specified type.
    */
   public IgelViewKurmas(IGameState gameState, Map<String, Artist> map) {

      gameBoard = new GameBoard(gameState, map);

      //
      // Set up instruction panel
      //
      JPanel instructionPanel = new JPanel();
      instructionLabel = new JLabel();
      instructionPanel.add(instructionLabel);

      //
      // Set up player panel
      //
      JPanel playerPanel = new JPanel();
      playerPanel.setLayout(new FlowLayout());
      for (int i = 0; i < gameState.numPlayers(); i++) {
         playerPanel.add(new JLabel("Player " + (i + 1) + ":"));
         playerPanel.add(new CirclePanel(Artist.HEDGEHOG_COLORS[i]));
         playerPanel.add(new JLabel("     "));
      }

      JPanel bottomPanel = new JPanel();
      bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.Y_AXIS));
      bottomPanel.add(playerPanel);
      bottomPanel.add(instructionPanel);


      //
      // Set up sideways panel
      //
      JPanel sidePanel = new JPanel();
      sidePanel.setLayout(new BoxLayout(sidePanel, BoxLayout.Y_AXIS));
      die = new GVDiePanel();
      sidePanel.add(die);
      sidePanel.add(passSidewaysMoveButton);

      // Initialize frame
      frame = new JFrame("Igel \u00c4rgern");
      frame.add(gameBoard, BorderLayout.CENTER);
      frame.add(bottomPanel, BorderLayout.SOUTH);
      frame.add(sidePanel, BorderLayout.EAST);
      frame.setSize(650, 650);
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setVisible(true);
   }

   /**
    * Constructor that uses a default artist map.
    *
    * @param gameState object describing the state of the game
    */
   public IgelViewKurmas(IGameState gameState) {
      this(gameState, IgelViewKurmas.defaultArtistMap());
   }


   // From IIgelView
   public void addCellActionListener(CellActionListener cellActionListener) {
      gameBoard.addCellActionListener(cellActionListener);
   }

   // From IIgelView
   public void addMouseListener(int row, int col, MouseListener listener) {
      gameBoard.addMouseListener(row, col, listener);
   }

   // From IIgelView
   public void update(IGameState state) {
      frame.repaint();
   }

   // From IIgelView
   public void addPassSidewaysMovePressedListener(ActionListener actionListener) {
      passSidewaysMoveButton.addActionListener(actionListener);
   }

   // From IIgelView
   public void enableSidewaysViewButton(boolean enable) {
      passSidewaysMoveButton.setEnabled(enable);
   }

   // From IIgelView
   public void displayDialogMessage(String message, String title) {
      JOptionPane.showMessageDialog(frame, message, title,
            JOptionPane.INFORMATION_MESSAGE);
   }

   /**
    * Close the view (i.e., dispose of the underlying {@code JFrame}.
    */
   public void close() {
      frame.setVisible(false);
      frame.dispose();
   }

   // From IIgelView
   public void addMenu(JMenuBar menu) {
      frame.setJMenuBar(menu);
   }

   /**
    * Mark this cell as the starting point of a proposed move and suppress the drawing of this piece (because it is
    * drawn translucently in the proposed target square).
    *
    * @param row    the row
    * @param column the column
    * @throws IllegalArgumentException if either {@code row} or {@code column} is not valid
    */
   public void setProposedMoveSource(int row, int column) {
      gameBoard.setProposedMoveSource(row, column);
   }

   /**
    * Specify that the user has proposed moving a hedgehog into the given cell and draw a translucent piece in the specified square.
    *
    * @param row    the row
    * @param column the column
    * @param player the player number
    * @throws IllegalArgumentException if either {@code row} or {@code column} are out of range.
    */
   public void setProposedMoveTarget(int row, int column, int player) {
      gameBoard.setProposedMoveTarget(row, column, player);
   }

   // From IIgelView
   public void clearProposedMove() {
      gameBoard.clearProposedMove();
   }

   // From IIgelView
   public void updateMessage(String newMessage) {
      instructionLabel.setText(newMessage);
      instructionLabel.repaint();
   }

   // From IIgelView
   public void updateDie(int newValue) {
      die.rollTo(newValue);
   }

   /**
    * Return the default  mapping of cell types to {@link Artist}s.
    *
    * @return the default  mapping of cell types to {@link Artist}s.
    */
   public static Map<String, Artist> defaultArtistMap() {
      Map<String, Artist> artistMap = new HashMap<String, Artist>();
      artistMap.put("Cell", new CellPanelArtist());
      artistMap.put("ConcreteBlock", new ConcreteBlockArtist());
      artistMap.put("FinishCell", new FinishCellArtist());
      artistMap.put("DeepPit", new DeepPitArtist());
      return artistMap;
   }
}
